@@ -22,7 +22,7 @@
 
 #include "metadatabase.h"
 
-#include <kmainwindow.h>
+#include <kparts/dockmainwindow.h>
 #include <kdockwidget.h>
 
 #include <qmap.h>
@@ -41,6 +41,7 @@ class KActionCollection;
 class KProcess;
 class KRecentFilesAction;
 class KToggleAction;
+class KToolBar;
 class KURL;
 class PropertyEditor;
 class QCloseEvent;
@@ -59,8 +60,12 @@ class QToolBar;
 #endif
 class Preferences;
 
+namespace KParts {
+  class PartManager;
+}
 
-class MainWindow : public KDockMainWindow
+
+class MainWindow : public KParts::DockMainWindow
 {
     Q_OBJECT
 
@@ -124,7 +129,8 @@ public slots:
     void showProperties( QObject *w );
     void updateProperties( QObject *w );
     void showDialogHelp();
-
+    void slotActivePartChanged(KParts::Part * );
+    
 signals:
     void addedFormFile(FormFile *);
     void removedFormFile(FormFile *);
@@ -177,9 +183,11 @@ public slots:
     void editFormSettings();
     void editPreferences();
     void editShortcuts();
+    void editPlugins();
     void editExternalTool(int id);
     void editToolExited(KProcess*);
     void editToolOutput(KProcess*, char*, int);
+    void configureEditor();
 
     void runForm();
     
@@ -271,6 +279,7 @@ private:
     QPopupMenu *windowMenu, *fileMenu;
 
     QToolBar *layoutToolBar;
+    KToolBar *fileTb;
 
     Preferences *prefDia;
     QMap<QString,QString> propertyDocumentation;
@@ -292,6 +301,7 @@ private:
     // Paths of external Kommander tools for Editor
     QStringList m_editorTools;
     QString m_toolOutput;
+    KParts::PartManager *m_partManager;
 
 public:
     QString lastSaveFilter;
