@@ -1,7 +1,7 @@
 /***************************************************************************
                     functionlib.cpp - Standard library of functions
                              -------------------
-    copyright          : (C) 2004      Michal Rudolf <mrudolf@kdewebdwev.org>
+    copyright          : (C) 2004-2006      Michal Rudolf <mrudolf@kdewebdwev.org>
     
  ***************************************************************************/
 
@@ -57,7 +57,7 @@ static ParseNode f_stringFind(Parser*, const ParameterList& params)
 
 static ParseNode f_stringFindRev(Parser*, const ParameterList& params)
 {
-  return params[0].toString().find(params[1].toString(), 
+  return params[0].toString().findRev(params[1].toString(), 
     params.count() == 3 ? params[2].toInt() : params[0].toString().length());
 }
 
@@ -90,23 +90,23 @@ static ParseNode f_stringLower(Parser*, const ParameterList& params)
 {
   return params[0].toString().lower();
 }
-    
+
 static ParseNode f_stringUpper(Parser*, const ParameterList& params)
 {
   return params[0].toString().upper();
 }
-  
+
 static ParseNode f_stringIsEmpty(Parser*, const ParameterList& params)
 {
   return params[0].toString().isEmpty();
 }
-    
+
 static ParseNode f_stringSection(Parser*, const ParameterList& params)
 {
   return params[0].toString().section(params[1].toString(), params[2].toInt(), 
     params.count() == 4 ? params[3].toInt() : params[2].toInt());
 }
-  
+
 static ParseNode f_stringArgs(Parser*, const ParameterList& params)
 {
   if (params.count() == 2)
@@ -147,6 +147,7 @@ static ParseNode f_echo(Parser*, const ParameterList& params)
 {
   for (uint i=0; i<params.count(); i++)
     std::cout << params[i].toString();
+  fflush(stdout);
   return ParseNode();
 }
 
@@ -156,7 +157,7 @@ static ParseNode f_echo(Parser*, const ParameterList& params)
 static ParseNode f_fileRead(Parser*, const ParameterList& params)
 {
   QFile file(params[0].toString());
-  if (!file.open(IO_ReadOnly))
+  if (!file.exists() || !file.open(IO_ReadOnly))
     return ParseNode("");
   QTextStream text(&file);
   return text.read();
@@ -164,7 +165,10 @@ static ParseNode f_fileRead(Parser*, const ParameterList& params)
 
 static ParseNode f_fileWrite(Parser*, const ParameterList& params)
 {
-  QFile file(params[0].toString());
+  QString fname = params[0].toString();
+  if (fname.isEmpty())
+    return 0;
+  QFile file(fname);
   if (!file.open(IO_WriteOnly))
     return 0;
   QTextStream text(&file);
@@ -175,7 +179,10 @@ static ParseNode f_fileWrite(Parser*, const ParameterList& params)
 
 static ParseNode f_fileAppend(Parser*, const ParameterList& params)
 {
-  QFile file(params[0].toString());
+  QString fname = params[0].toString();
+  if (fname.isEmpty())
+    return 0;
+  QFile file(fname);
   if (!file.open(IO_WriteOnly | IO_Append))
     return 0;
   QTextStream text(&file);
@@ -189,7 +196,7 @@ static ParseNode f_fileAppend(Parser*, const ParameterList& params)
 
 
 /******************* DCOP function ********************************/
-static ParseNode f_dcop(Parser* parser, const ParameterList& params)
+static ParseNode f_internalDcop(Parser* parser, const ParameterList& params)
 {
   SpecialFunction function = SpecialInformation::functionObject("DCOP", params[0].toString());
   int functionId = SpecialInformation::function(Group::DCOP, params[0].toString());
@@ -217,45 +224,49 @@ static ParseNode f_dcop(Parser* parser, const ParameterList& params)
 }
 
 
-static ParseNode f_externalDcop(Parser*, const ParameterList& params)
+static ParseNode f_dcop(Parser*, const ParameterList& params)
 {
-  QCString appId = kapp->dcopClient()->appId();
-  QCString object = "KommanderIf";
-  SpecialFunction function = SpecialInformation::functionObject("DCOP", params[0].toString());
-  
-  if (!function.isValidArg(params.count() - 1))
-    return ParseNode();
-  
+  QCString appId = params[0].toString().latin1();
+  QCString object = params[1].toString().latin1();
+  QString function = params[2].toString().section('(', 0, 0);
+  QStringList items = QStringList::split(",", params[2].toString().section('(', 1, 1).section(')', 0, 0));
   QByteArray byteData;
   QDataStream byteDataStream(byteData, IO_WriteOnly);
-  for (uint i=0 ; i<params.count()-1; i++) 
+
+  if (items.count() != params.count() - 3)
   {
-    if (function.argumentType(i) == "int")
-      byteDataStream << params[i+1].toInt();
-    else if (function.argumentType(i) == "long")
-      byteDataStream << params[i+1].toInt();
-    else if (function.argumentType(i) == "float")
-      byteDataStream << params[i+1].toDouble();
-    else if (function.argumentType(i) == "double")
-      byteDataStream << params[i+1].toDouble();
-    else if (function.argumentType(i) == "bool")
-      byteDataStream << (bool)params[i+1].toInt();
-    else if (function.argumentType(i) == "QStringList")
-      if (params[i].toString().find('\n') != -1)
-        byteDataStream << QStringList::split("\n", params[i+1].toString(), true);
-      else
-        byteDataStream << QStringList::split("\\n", params[i+1].toString(), true);
-    else 
-      byteDataStream << params[i+1].toString();
+     qDebug("Wrong number of parameters");
+     return ParseNode();
   }
-  
+  int i = 3;
+  for (QStringList::Iterator it = items.begin(); it != items.end(); ++it)
+  {
+    *it = (*it).stripWhiteSpace();
+    if (*it == "int")
+      byteDataStream << params[i++].toInt();
+    else if (*it == "long")
+      byteDataStream << params[i++].toInt();
+    else if (*it == "float")
+      byteDataStream << params[i++].toDouble();
+    else if (*it == "double")
+      byteDataStream << params[i++].toDouble();
+    else if (*it == "bool")
+      byteDataStream << (bool)params[i++].toInt();
+    else if (*it == "QStringList")
+      if (params[i].toString().find('\n') != -1)
+        byteDataStream << QStringList::split("\n", params[i++].toString(), true);
+      else
+        byteDataStream << QStringList::split("\\n", params[i++].toString(), true);
+    else 
+      byteDataStream << params[i++].toString();
+  }
+  function.append(QString("(%1)").arg(items.join(",")));
   QCString replyType, byteReply;
   DCOPClient* cl = KApplication::dcopClient();
-  if (!cl || !cl->call(appId, object, function.prototype(SpecialFunction::NoSpaces).latin1(), 
+  if (!cl || !cl->call(appId, object, function.latin1(),
        byteData, replyType, byteReply))
   {
-    //printError(i18n("Tried to perform DCOP query, but failed."));
-    qDebug("Failure");
+    qDebug("DCOP failure");
     return ParseNode();
   }
   QDataStream byteReplyStream(byteReply, IO_ReadOnly);
@@ -285,7 +296,7 @@ static ParseNode f_externalDcop(Parser*, const ParameterList& params)
   }
   else if(replyType != "void")
   {
-  //printError(i18n("DCOP return type %1 is not yet implemented.").arg(replyType.data()));
+     qDebug("%s", QString("DCOP return type %1 is not yet implemented.").arg(replyType.data()).latin1());
   }
 
   return ParseNode();
@@ -335,15 +346,15 @@ static ParseNode f_arrayKeys(Parser* P, const ParameterList& params)
     return ParseNode();
   return QStringList(P->array(params[0].toString()).keys()).join("\n");
 }
-    
+
 static ParseNode f_arrayValues(Parser* P, const ParameterList& params)
 {
   if (!P->isArray(params[0].toString()))
     return ParseNode();
   QValueList<ParseNode> values = P->array(params[0].toString()).values(); 
   QString array;
-  for (QValueList<ParseNode>::Iterator it = values.begin(); it != values.end(); ++it ) 
-    array += (*it).toString();
+  for (QValueList<ParseNode>::ConstIterator it = values.begin(); it != values.end(); ++it ) 
+    array += (*it).toString() + '\n';
   return array;
 }
 
@@ -406,7 +417,7 @@ static ParseNode f_inputText(Parser*, const ParameterList& params)
     value = params[0].toString();
   return KInputDialog::getText(params[0].toString(), params[1].toString(), value);
 }
-    
+
 static ParseNode f_inputPassword(Parser*, const ParameterList& params)
 {
   QCString value;
@@ -415,7 +426,7 @@ static ParseNode f_inputPassword(Parser*, const ParameterList& params)
   KPasswordDialog::getPassword(value, params[0].toString());
   return QString::fromLocal8Bit(value);
 }
-    
+
 static ParseNode f_inputValue(Parser*, const ParameterList& params)
 {
   return KInputDialog::getInteger(params[0].toString(), params[1].toString(), 
@@ -423,7 +434,7 @@ static ParseNode f_inputValue(Parser*, const ParameterList& params)
                                   params.count() > 5 ? params[5].toInt() : 1,
                                   (bool*)0);
 }
-  
+
 static ParseNode f_inputValueDouble(Parser*, const ParameterList& params)
 {
   return KInputDialog::getDouble(params[0].toString(), params[1].toString(), 
@@ -623,7 +634,8 @@ void ParserData::registerStandardFunctions()
   registerFunction("file_read", Function(&f_fileRead, ValueString, ValueString, 1, 1));
   registerFunction("file_write", Function(&f_fileWrite, ValueInt, ValueString, ValueString, 2, 100));
   registerFunction("file_append", Function(&f_fileAppend, ValueInt, ValueString, ValueString, 2, 100));
-  registerFunction("dcop", Function(&f_dcop, ValueString, ValueString, ValueString, 2, 100));
+  registerFunction("internalDcop", Function(&f_internalDcop, ValueString, ValueString, ValueString, 2, 100));
+  registerFunction("dcop", Function(&f_dcop, ValueString, ValueString, ValueString, 3, 100));
   registerFunction("exec", Function(&f_exec, ValueString, ValueString, ValueString, 1, 2));
   registerFunction("i18n", Function(&f_i18n, ValueString, ValueString));
   registerFunction("env", Function(&f_env, ValueString, ValueString));
