@@ -26,11 +26,12 @@
 
 /* OTHER INCLUDES */
 #include <kommanderwidget.h>
+#include <kommander_export.h>
 
 class QWidget;
 
 class QShowEvent;
-class ButtonGroup : public QButtonGroup, public KommanderWidget
+class KOMMANDER_EXPORT ButtonGroup : public QButtonGroup, public KommanderWidget
 {
   Q_OBJECT
 
