@@ -616,7 +616,8 @@ xslDbgCd(xsltTransformContextPtr styleCtxt, xmlShellCtxtPtr ctxt,
         } else {
             xsldbgGenericErrorFunc(i18n("Error: XPath %1 was not found.\n").arg(xsldbgText(arg)));
         }
-        ctxt->pctxt->node = NULL;
+        if (ctxt->pctxt)
+          ctxt->pctxt->node = NULL;
     }
     return result;
 }
@@ -1988,7 +1989,7 @@ shellPrompt(xmlNodePtr source, xmlNodePtr doc, xmlChar * filename,
 		    cmdResult = 0;
 		}
 #else
-	        xsldbgGenericErrorFunc(i18n("Warning: The %1 command is disbled\n").arg("lang"));
+	        xsldbgGenericErrorFunc(i18n("Warning: The %1 command is disabled\n").arg("lang"));
 		cmdResult = 1;
 #endif
 		break;
