@@ -27,6 +27,8 @@
 #include "structtreetag.h"
 #include "kafkacommon.h"
 
+QMap<Node*, int> nodes; //list of all created nodes. Used to do some own memory management and avoid double deletes, for whatever reason they happen...
+
 int NN = 0; //for debugging purposes: count the Node objects
 
 GroupElementMapList globalGroupMap;
@@ -46,11 +48,35 @@ Node::Node(Node *parent)
   m_leafNode = 0L;
   m_groupElements.clear();
   NN++;
+//   if (nodes.contains(this) == 0)
+  nodes[this] = 1;
+//   else
+//   {
+//     kdError(24000) << "A node with this address " << this << " already exists!" << endl; 
+//   }
 }
 
+bool Node::deleteNode(Node *node)
+{
+  if (!node)
+    return true;
+  if (!nodes.contains(node))
+  {     
+    kdDebug(24000) << "Trying to delete a node with address " << node << " that was not allocated!" << endl; 
+    return false;
+  }
+  delete node;
+  return true;
+}
 
 Node::~Node()
 {
+//   if (!nodes.contains(this))
+//   {     
+//     kdError(24000) << "No node with this address " << this << " was allocated!" << endl; 
+//     return;
+//   }
+    
   //It has no use, except to know when it crash why it has crashed.
   //If it has crashed here, the Node doesn't exist anymore.
   // If it has crashed the next line, it is a GroupElements bug.
@@ -60,16 +86,23 @@ Node::~Node()
     tag->setCleanStrBuilt(false);
 
   detachNode();
+  nodes.erase(this);
   if (prev && prev->next == this)
       prev->next = 0L;
   if (parent && parent->child == this)
       parent->child = 0L;
   if (removeAll)
   {
-    delete child;
+    deleteNode(child);
     child = 0L;
-    delete next;
+    deleteNode(next);
     next = 0L;
+  } else
+  {
+    if (next && next->prev == this)
+      next->prev = 0L;
+    if (child && child->parent == this)
+      child->parent = 0L;
   }
 
   delete tag;
@@ -487,6 +520,12 @@ void Node::operator =(Node* node)
 
 void Node::detachNode()
 {
+  if (nodes.contains(this) == 0)
+  {
+    kdError(24000) << "No node with this address " << this << " was allocated!" << endl; 
+    return;
+  }    
+
   int count = 0;
   //kdDebug(24000) << &m_groupElements << " " << this << endl;
   //Remove the references to this node from the list of group elements.
