@@ -32,7 +32,7 @@
 #include "quantacommon.h"
 #include "resource.h"
 
-//#undef DEBUG_PARSER
+//#define DEBUG_PARSER
 
 SAParser::SAParser()
 {
@@ -209,9 +209,11 @@ bool SAParser::slotParseOneLine()
                   ParserCommon::appendAreaToTextNode(m_write, s_currentContext.area, s_currentNode);
                 else
                   s_currentNode = ParserCommon::createTextNode(m_write, s_currentNode, s_line, groupKeywordPos, s_currentContext.parentNode);
-                s_currentNode->insideSpecial = true;
-                s_currentNode->specialInsideXml = m_specialInsideXml;
-
+                if (s_currentNode)
+                {
+                  s_currentNode->insideSpecial = true;
+                  s_currentNode->specialInsideXml = m_specialInsideXml;
+                }
                 s_previousContext = s_contextStack.pop();
                 s_currentContext.parentNode = s_previousContext.parentNode;
                 s_currentContext.lastNode = s_previousContext.lastNode;
@@ -343,8 +345,11 @@ bool SAParser::slotParseOneLine()
             {
               s_currentNode = ParserCommon::createTextNode(m_write, s_currentNode, s_line, areaEndPos, s_parentNode);
             }
-            s_currentNode->insideSpecial = true;
-            s_currentNode->specialInsideXml = m_specialInsideXml;
+            if (s_currentNode)
+            {
+              s_currentNode->insideSpecial = true;
+              s_currentNode->specialInsideXml = m_specialInsideXml;
+            }
           }
           //kdDebug(24000) << QString("Special area %1 ends at %2, %3").arg(s_dtd->name).arg(s_line).arg(lastCol) << endl;
 
@@ -485,10 +490,20 @@ bool SAParser::slotParseOneLine()
           p = s_textLine.find(s_currentContext.startString, p);
           if (p != -1)
           {
-            if (p == 0 || (p > 0 && s_textLine[p-1] != '\\'))
+            if (p >= 0)
             {
-              pos = p;
-              break;  
+              int i = p - 1;
+              int slahNum = 0;
+              while (i > 0 && s_textLine[i] == '\\')
+              {
+                slahNum++;
+                i--;
+              }
+              if (p == 0 || (slahNum % 2 == 0))
+              {
+                pos = p;
+                break;
+              }
             }
           } else
             break;
@@ -796,30 +811,18 @@ Node *SAParser::parsingDone()
       s_currentNode = n;
     }
     Node *g_node = n;
-    Node *g_endNode = 0L;
 #ifdef DEBUG_PARSER
     kdDebug(24001) << "Calling slotParseForScriptGroup from parsingDone. Synch:" << m_synchronous << endl;
 #endif
     //parse for groups only when doing aynchronous detailed parsing
     if (!m_synchronous)
     {
-      bool parsingLastNode = true;
-      Node *n = g_endNode;
-      while (n)
-      {
-        n = n->nextSibling();
-        if (n && n->insideSpecial)
-        {
-          parsingLastNode = false;
-          break;
-        }
-      }
-      SAGroupParser *groupParser = new SAGroupParser(this, write(), g_node, g_endNode, m_synchronous, parsingLastNode, true);
+      SAGroupParser *groupParser = new SAGroupParser(this, write(), g_node, 0L, m_synchronous, true /*last node*/, true);
       connect(groupParser, SIGNAL(rebuildStructureTree(bool)), SIGNAL(rebuildStructureTree(bool)));
       connect(groupParser, SIGNAL(cleanGroups()), SIGNAL(cleanGroups()));
       connect(groupParser, SIGNAL(parsingDone(SAGroupParser*)), SLOT(slotGroupParsingDone(SAGroupParser*)));
       groupParser->slotParseForScriptGroup();
-      m_groupParsers.append(groupParser);      
+      m_groupParsers.append(groupParser);
      }
   }
 
@@ -892,12 +895,13 @@ void SAParser::slotParseNodeInDetail()
         m_currentNode->tag->type != Tag::Text &&
         m_currentNode->tag->type != Tag::Empty)
     {
-      delete m_currentNode->child;
+      Node::deleteNode(m_currentNode->child);
       m_currentNode->child = 0L;
       AreaStruct area(m_currentNode->tag->area());
       s_next = 0L;
       m_useNext = false;
-      if (m_currentNode->next)
+      //FIXME: Find out why can the tag become 0L
+      if (m_currentNode->next && m_currentNode->next->tag)
       {
         AreaStruct area2(m_currentNode->next->tag->area());
         area.eLine = area2.eLine;
@@ -909,7 +913,7 @@ void SAParser::slotParseNodeInDetail()
           Node *secondNext = m_currentNode->next->next;
           if (secondNext)
             secondNext->prev = m_currentNode;
-          delete m_currentNode->next;
+          Node::deleteNode(m_currentNode->next);
           m_currentNode->next = secondNext;
           m_useNext = true;
         }
